//
//  ParagraphStyle.js
//  Kite
//
//  Created by Joe Rhodes
//  Copyright (c) 2016 Joe Rhodes. All rights reserved.
//


// -------------------------------------------------------------------------------------------
// Constants
// -------------------------------------------------------------------------------------------

var kNSParagraphStyleClassName          = "NSParagraphStyle"
var kJRClassNameKey                     = "className"
var kJRLineSpacingKey                   = "lineSpacing"
var kJRParagraphSpacingKey              = "paragraphSpacing"
var kJRParagraphSpacingBeforeKey        = "paragraphSpacingBefore"
var kJRMaximumLineHeightKey             = "maximumLineHeight"
var kJRMinimumLineHeightKey             = "minimumLineHeight"
var kJRAlignmentKey                     = "alignment"
var kJRHeadIndentKey                    = "headIndent"
var kJRTailIndentKey                    = "tailIndent"
var kJRFirstLineHeadIndentKey           = "firstLineHeadIndent"
var kJRLineBreakMode                    = "lineBreakMode"
var kJRHyphenationFactorKey             = "hyphenationFactor"
var kJRTighteningFactorForTruncationKey = "tighteningFactorForTruncation"
var kJRLineHeightMultiple               = "lineHeightMultiple"


// -------------------------------------------------------------------------------------------
// Paragraph Style
// -------------------------------------------------------------------------------------------

ParagraphStyle = function(paraStyle) {

    this.lineSpacing = paraStyle.lineSpacing()
    this.paragraphSpacing = paraStyle.paragraphSpacing()
    this.alignment = paraStyle.alignment()
    this.headIndent = paraStyle.headIndent()
    this.tailIndent = paraStyle.tailIndent()
    this.firstLineHeadIndent = paraStyle.firstLineHeadIndent()
    this.minimumLineHeight = paraStyle.minimumLineHeight()
    this.maximumLineHeight = paraStyle.maximumLineHeight()
    this.paragraphSpacingBefore = paraStyle.paragraphSpacingBefore()
    this.hyphenationFactor = paraStyle.hyphenationFactor()
    this.tighteningFactorForTruncation = paraStyle.tighteningFactorForTruncation()
    this.lineHeightMultiple = paraStyle.lineHeightMultiple()
}

ParagraphStyle.prototype = Object.create(Object.prototype, {

    // -------------------------------------------------------------------------------------------
    // Archiving
    // -------------------------------------------------------------------------------------------

    archivableRepresentation: {
        value: function() {

            var arch = {}

            arch[kJRClassNameKey] = kNSParagraphStyleClassName
            arch[kJRLineSpacingKey] = this.lineSpacing
            arch[kJRParagraphSpacingKey] = this.paragraphSpacing
            arch[kJRAlignmentKey] = this.alignment
            arch[kJRHeadIndentKey] = this.headIndent
            arch[kJRTailIndentKey] = this.tailIndent
            arch[kJRFirstLineHeadIndentKey] = this.firstLineHeadIndent
            arch[kJRMinimumLineHeightKey] = this.minimumLineHeight
            arch[kJRMaximumLineHeightKey] = this.maximumLineHeight
            arch[kJRParagraphSpacingBeforeKey] = this.paragraphSpacingBefore
            arch[kJRHyphenationFactorKey] = this.hyphenationFactor
            arch[kJRTighteningFactorForTruncationKey] = this.tighteningFactorForTruncation
            arch[kJRLineHeightMultiple] = this.lineHeightMultiple

            return arch
        }
    }
})
