//
//  BezierPathWriter.js
//  Kite
//
//  Created by Joe Rhodes
//  Copyright (c) 2016 Joe Rhodes. All rights reserved.
//

// Takes a MSShapePath to write
//


@import 'Helpers.js'


ShapePathWriter = function(kiteLayer) {

    this.kiteLayer = kiteLayer
    this.shapePath = kiteLayer.path
    this.sketchLayer = kiteLayer.sketchLayer
    this.exportInfo = kiteLayer.exportInfo

    this.fillRule = "non-zero"
}

ShapePathWriter.prototype = Object.create(Object.prototype, {

    svgPathString: {
        value: function() {

            var pathString = this.pathStringFromSVGExport()

            if (pathString != null) {
                return pathString
            }
            else {
                return this.pathStringFromRawBezierPath()
            }
        }
    },

    // -------------------------------------------------------------------------------------------
    // First try to export the layer as an SVG file and extract the <path>'s "d" attribute
    // -------------------------------------------------------------------------------------------

    pathStringFromSVGExport: {
        value: function() {

            var pathString = null

            var pathForExportedSVGFile = this.pathForExportedSVGFile()
            var fileManager = NSFileManager.defaultManager()
            var exportedSVGFileExists = fileManager.fileExistsAtPath(pathForExportedSVGFile)

            if ( ! exportedSVGFileExists ) {
                return null
            }

            var svgFileContents = NSString.stringWithContentsOfFile_encoding_error(pathForExportedSVGFile, NSUTF8StringEncoding, null)

            pathString = this.pathDElementFromSVGTextString(svgFileContents)

            // Try to get the fillRule while we have the SVG string representation
            //
            this.fillRule = (this.fillRuleFromSVGTextString(svgFileContents)) == "evenodd" ? "even-odd" : "non-zero"

            return pathString
        }
    },

    pathDElementFromSVGTextString: {
        value: function(svgString) {

            try {
                var dAttrIndex = svgString.indexOf(" d=")

                if (dAttrIndex == -1) { return null }

                var openQuoteIndex = svgString.indexOf('"', dAttrIndex)

                if (openQuoteIndex == -1) { return null }

                var nextQuoteIndex = svgString.indexOf('"', openQuoteIndex + 1)

                if (nextQuoteIndex == -1) { return null }

                var substring = svgString.substring(openQuoteIndex + 1, nextQuoteIndex)

                return substring
            }
            catch(error) {

                log(error)
                return null
            }
        }
    },

    fillRuleFromSVGTextString: {
        value: function(svgString)   {
            try {
                var fillAttrIndex = svgString.indexOf(" fill-rule=")

                if (fillAttrIndex == -1) { return null }

                var openQuoteIndex = svgString.indexOf('"', fillAttrIndex)

                if (openQuoteIndex == -1) { return null }

                var nextQuoteIndex = svgString.indexOf('"', openQuoteIndex + 1)

                if (nextQuoteIndex == -1) { return null }

                var substring = svgString.substring(openQuoteIndex + 1, nextQuoteIndex)

                return substring
            }
            catch(error) {

                log(error)
                return null
            }
        }
    },

    pathForExportedSVGFile: {
        value: function() {

            var kiteLayer = this.kiteLayer
            var tempSVGFilePath = this.tempSVGFileURL()
            var sketchDoc = this.exportInfo.sketchDocument
            var sketchPage = this.exportInfo.sketchPage
            var layerSlice = kiteLayer.artboardOrSliceForLayer(sketchPage)
            var svgFilePath = tempSVGFilePath.path()

            sketchDoc.saveArtboardOrSlice_toFile(layerSlice, svgFilePath)

            return svgFilePath
        }
    },

    tempSVGFileURL: {
        value: function() {

            var fileUUID = NSUUID.UUID().UUIDString()
            var tempURL = Helpers.tempSVGExportURL().URLByAppendingPathComponent(fileUUID)
            tempURL = tempURL.URLByAppendingPathExtension("svg")

            return tempURL
        }
    },

    // -------------------------------------------------------------------------------------------
    // We may need to fallback to just grabbing the raw SVG path attribute from the bezier path
    // -------------------------------------------------------------------------------------------

    pathStringFromRawBezierPath: {
        value: function() {

            var shapePath = this.shapePath;

            if ( ! shapePath ) {
                return ""
            }

            var pathString = ""
            var bezierPath = NSBezierPath.bezierPathWithPath(shapePath);

            if (typeof bezierPath.svgPathAttribute != 'function') { return "" }

            var svgPathAttribute = bezierPath.svgPathAttribute()

            if (svgPathAttribute && svgPathAttribute.name() == "d") {

                pathString = svgPathAttribute.stringValue()
            }

            return pathString
        }
    },
})
